// Version with additionally transmitting continous carrier between 
// CW sequences for 3 to 60 seconds according to #A3 setting 

/* A simple Morse code sender for the HopeRF rfm69hcw on a ProMini 3.3
 * by Charles Webb ( KC1ENN@arrl.net ); Mark VandeWettering ( k6hx@arrl.net )
 * with extensions by Klaus Hirschelmann ( dj7oo@t-online.de ) */
 
#include <SPI.h>

#define ssPin 10       // SS  Chip select pin for reading/writing to the rf69 registers with SPI
#define MY_DELAY 1000  // 1 second delay between loops, sending the message
#define N_MORSE (sizeof(morsetab)/sizeof(morsetab[0]))
#define SPEED (15)     // WORKS 5 - 50 wpm FL-DIGI
#define DOTLEN (1200/SPEED)
#define DASHLEN (3*(1200/SPEED))
#define MY_MESSAGE_1 "   TEST TEST TEST DE DJ7OO DJ7OO DJ7OO IN JN49CX JN49CX JN49CX ARSK   "
#define MY_MESSAGE_2 "   CQ CQ CQ DE DJ7OO DJ7OO DJ7OO IN JN49CX JN49CX JN49CX ARSK   "
#define MY_TX_POWER 0x5F  // PA0 only range 0x50 -18dBm to 0x5F -3dBm
                          // measured: +13dBm   

/////////////////////////////////////////////////////////////////////////////////
void setupSPI() {
  SPI.begin();
  SPI.setBitOrder(MSBFIRST);
  SPI.setClockDivider(SPI_CLOCK_DIV4);
}

///////////////////////////////////////////////////////////////////////////////////
void setup() {
  Serial.begin(9600);             // baud speed for sending to the arduino serial monitor
  Serial.println("Starting up");  // print to the serial monitor
  pinMode(ssPin, OUTPUT);         // define
  pinMode(A0, INPUT_PULLUP);
  pinMode(A1, INPUT_PULLUP);
  pinMode(A2, INPUT_PULLUP);
  pinMode(8, OUTPUT);
  
  setupSPI();
  writeReg(0x02,0x34);            // Modulation for OOK, Continuous w.o bit synch, OOK, no shaping
  printReg(0x02);
  
// multiply desired frequency(MHz) by 16384; convert result to HEX; divide into 3 blocks with 2 characters each
// sample: 432.390*16384=7084277.76; 7084277.76 in dec = 6C18F5 in hex; divide into 3 hex blocks: 0x6C, 0x18, 0xF5 
                               
  writeReg(0x07,0x6C);  // 6C,18,F5 defaults to a frequency of approximately 432.390 MHZ with crystal tolerance 
  writeReg(0x08,0x18);  
  writeReg(0x09,0xF5);   
    
   printReg(0x07);
   printReg(0x08);
   printReg(0x09); 
// writeReg(0x26,0x07); // CLK off to save power by not enableing an external clock signal on a DIO port.
// printReg(0x26);
   writeReg(0x01,0x0C);  // Set mode to Transmitter TX mode
   printReg(0x01);
}

/////////////////////////////////////////
struct t_mtab { char c, pat; } ;
struct t_mtab morsetab[] = {
  {'.', 106},
  {',', 115},
  {'?', 76},
  {'/', 41},
  {'A', 6},
  {'B', 17},
  {'C', 21},
  {'D', 9},
  {'E', 2},
  {'F', 20},
  {'G', 11},
  {'H', 16},
  {'I', 4},
  {'J', 30},
  {'K', 13},
  {'L', 18},
  {'M', 7},
  {'N', 5},
  {'O', 15},
  {'P', 22},
  {'Q', 27},
  {'R', 10},
  {'S', 8},
  {'T', 3},
  {'U', 12},
  {'V', 24},
  {'W', 14},
  {'X', 25},
  {'Y', 29},
  {'Z', 19},
  {'1', 62},
  {'2', 60},
  {'3', 56},
  {'4', 48},
  {'5', 32},
  {'6', 33},
  {'7', 35},
  {'8', 39},
  {'9', 47},
  {'0', 63}
} ;

//////////////////////////////////////////////////////////////////////////////////
void printReg(byte data) {
  Serial.print("Register "); 
  Serial.print(data); 
  Serial.print(" = "); 
  Serial.println(readReg(data), HEX);
}

//////////////////////////////////////////////////////////////////////////////////
void writeReg(uint8_t addr, uint8_t value) {
  digitalWrite(ssPin,LOW);
  SPI.transfer(addr | 0x80);
  SPI.transfer(value);
  digitalWrite(ssPin, HIGH);
}

/////////////////////////////////////////////////////////////////////////////////
uint8_t readReg(uint8_t addr) {
  digitalWrite(ssPin, LOW);
  SPI.transfer(addr & 0x7F);
  uint8_t regval = SPI.transfer(0);
  digitalWrite(ssPin, HIGH);
  return regval;
}

/////////////////////////////////////////////////////////////////////////////////
void txOn(int txTime) {
  writeReg(0x11,MY_TX_POWER);   // PA On  ***************
  delay(txTime);
  writeReg(0x11,0x00);          // PA Off ***************
}

////////////////////////////////////////////////////////////////////////////////
void dash() {
  txOn(DASHLEN);
  delay(DOTLEN);
}

/////////////////////////////////////////////////////////////////////////////////
void dit() {
  txOn(DOTLEN);
  delay(DOTLEN);
}

//////////////////////////////////////////////////////////////////////////////////
void send(char c) {
  int i ;
  if (c == ' ') {
    // Serial.print(c) ;
    delay(7*DOTLEN) ;
    return ;
  }
  for (i=0; i<N_MORSE; i++) {
    if (morsetab[i].c == c) {
      unsigned char p = morsetab[i].pat ;
      // Serial.print(morsetab[i].c) ;
      while (p != 1) {
          if (p & 1)
            dash() ;
          else
            dit() ;
          p = p / 2 ;
      }
      // tone(cwPin,500,2*DOTLEN);
      delay(2*DOTLEN) ;
      return ;
    }
  }
  /* if we drop off the end, then we send a space */
  // Serial.print("?") ;
}

//////////////////////////////////////////////////////////////////////////////////
void sendmsg(char *str) {
  while (*str)
    send(*str++) ;
  // Serial.println("");
}

///////////////////////////////////////////////////////////////////////
// variable delay depending on #A3 analog value 
   void sequence_delay()
 {
   int val,z,calc;
   val = analogRead(A3);  // read analog value from poti ( 0 to 1023 )
   calc = map(val,0,1023,3,60);
// calc = 3;                      // ************ only for testing ************* 
   Serial.print("for "),Serial.print(calc),Serial.println(" Secs.");
   for(z=1; z<(calc); z++)  
   { delay(1000); }         // delay = 3-60 Secs. (3 to 60 Secs.)
 }

 ////////////////////////////////////////////////////////////////////////////////
void loop() {
  Serial.println("Send CW sequence");
  digitalWrite(8, HIGH); 

   if(digitalRead(A0)==LOW)
 { sendmsg(MY_MESSAGE_1) ; 
   digitalWrite(8, LOW);
   delay(MY_DELAY); 
   Serial.print("Send continous carrier ");
   digitalWrite(8, HIGH); 
   writeReg(0x11,MY_TX_POWER); 
   sequence_delay();
   digitalWrite(8, LOW);
   writeReg(0x11,0x00); 
   delay(MY_DELAY); }
 
  else if(digitalRead(A1)==LOW)
  { sendmsg(MY_MESSAGE_2) ; 
   digitalWrite(8, LOW);
   delay(MY_DELAY); 
   Serial.print("Send continous carrier ");
   digitalWrite(8, HIGH); 
   writeReg(0x11,MY_TX_POWER); 
   sequence_delay();
   digitalWrite(8, LOW);
   writeReg(0x11,0x00); 
   delay(MY_DELAY); }
  
  else if(digitalRead(A2)==LOW)
  { writeReg(0x11,MY_TX_POWER); }
 }


